import time

from spikewrap.pipeline.full_pipeline import run_full_pipeline


config_name = "test_default"


def get_sessions_and_runs(mtd):
    sessions_and_runs = {
                         s.session_path_derivatives.stem: [
                            x.parent.stem for x in list(s.session_path_raw.rglob("*.ap.bin"))
        ]

        for s in mtd.sessions
    }
    if len(sessions_and_runs.values()) == 0:
        raise ValueError(f"no sessions and/or runs found: {sessions_and_runs}")
    return sessions_and_runs


def run_full_sorting_pipeline_on_mouse(mtd, sorter="kilosort2_5"):
    sessions_and_runs = get_sessions_and_runs(mtd)
    base_path = mtd.rawdata_directory.parent
    t = time.time()
    print("using new edits")

    for session, runs in sessions_and_runs.items():
        single_session_dict = {session: runs}
        run_full_pipeline(
            base_path=base_path,
            sub_name=mtd.mouse_id,
            sessions_and_runs=single_session_dict,
            data_format='spikeglx',
            config_name=config_name,
            sorter=sorter,
            concat_sessions_for_sorting=False,
            concat_runs_for_sorting=False,
            existing_preprocessed_data="skip_if_exists",  # this is kind of confusing...
            existing_sorting_output="skip_if_exists",
            overwrite_postprocessing=True,
            delete_intermediate_files=(),
            slurm_batch=True,
            save_preprocessing_chunk_size=12000,
        )

        print(f"TOOK {time.time() - t}")



import pathlib

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from looming_spots.loom_io.load import sync_raw_and_processed_data

from npix_lse.slurm.slurm_interfaces.bombcell import bombcell_slurm
from npix_lse.metadata.sorting_metadata import SortingMetadata
from npix_lse.slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def main():
    sub_ids = [
        "sub-001_id-T10",
        "sub-002_id-T11",
        "sub-003_id-T12",
        "sub-004_id-T13",
        # "sub-005_id-TS24",
        "sub-006_id-TS26",
        "sub-007_id-TS33",
        "sub-008_id-TS34",
    ]
    root = pathlib.Path("/ceph/margrie/slenzi/2023/msj_photometry/")
    rawdata_root = root / "rawdata"
    processed_data_root = root / "derivatives"

    for mouse_id in sub_ids:
        mtd = SortingMetadata(
            mouse_id,
            root=processed_data_root,
        )

        sync_raw_and_processed_data(mouse_id=mtd.mouse_id,
                                    raw_directory=rawdata_root,
                                    processed_directory=processed_data_root,
                                    )

        mtg = MouseLoomTrialGroup(mouse_id,
                                  processed_data_dir=str(processed_data_root),
                                  photometry=False,
                                  )
        track_mouse_slurm(mtd)


def bombcell_all_sessions(mtd):
    for s in mtd.sessions:
        if s.kilosort_dir:
            bombcell_slurm(s)


if __name__ == "__main__":
    main()



import pathlib



def voxel_sizes(recipe_path):
    import yaml

    with open(str(recipe_path), "r") as stream:
        try:
            params = yaml.safe_load(stream)
            return params["VoxelSize"]
        except yaml.YAMLError as exc:
            print(exc)


def get_brain_paths(
    mouse_id,
    brain_dir,
):
    p=pathlib.Path(brain_dir) / mouse_id / "stitchedImages_100"
    return list(p.glob("*"))


def brainreg_command(mouse_directory_derivatives, serial2p_directory_raw, function="brainreg", atlas="allen_mouse_10um", additional=None):
    input_paths = get_brain_paths(mouse_directory_derivatives.stem, serial2p_directory_raw)
    brainreg_commands = []
    for input_path in input_paths:
        output_path = mouse_directory_derivatives / "histology" / atlas / input_path.stem
        print(input_path)
        print(output_path)
        recipe_path = list(input_path.parent.parent.glob("recipe*"))[0]
        voxels = voxel_sizes(recipe_path)
        additional = f"--additional {input_path.parent / additional}" if additional is not None else ""
        cmd = f"{function} {input_path} {output_path} {additional} -v {voxels['Z']} {voxels['X']} {voxels['Y']} --orientation psr --atlas {atlas}"
        brainreg_commands.append(cmd)
    return brainreg_commands


def brainreg_commandline(
     mouse_directory_derivatives, serial2p_directory_raw, function="brainreg", atlas="kim_mouse_10um", logs_path=None, additional=None
):
    brainreg_commands = brainreg_command(mouse_directory_derivatives, serial2p_directory_raw,
                                         function=function, atlas=atlas, additional=additional)
    # subprocess.run(command,
    #                executable="/bin/bash",
    #                shell=True,
    #                )



from pathlib import Path

from bombcell.bombcell import write_bombcell_matlab_script
from pp_slurm import slurm_functions


def bombcell_slurm(s, overwrite=False):
    # This is a hack right now and passes the actually used command via the additional_string
    # command, the rest is more or less irrelevant.
    if not (s.kilosort_dir.parent / "bc_quality_metrics.m").exists() or overwrite:
        write_bombcell_matlab_script(s)
        kwarg_dict = {
            "slurm_batch": True,
            "logs_path": Path(f"{s.session_path_derivatives}") / "logs",
            "module_string": "matlab/R2022a",
            "additional_string": f'matlab -nodisplay -nosplash -nodesktop -r "run(\'{s.kilosort_dir.parent}/bc_quality_metrics.m\')"; exit;'
        }
        slurm_functions.run_bombcell_slurm(**kwarg_dict)
    else:
        print(f'session {s.session_path_derivatives} has been processed and overwrite set to: {overwrite}')




import warnings

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from looming_spots.loom_io.load import sync_raw_and_processed_data
from magicgui import magicgui

from npix_lse.slurm.slurm_interfaces.bombcell import bombcell_slurm
from npix_lse.slurm.slurm_interfaces.brainreg import brainreg_slurm
from npix_lse.metadata.sorting_metadata import SortingMetadata
from npix_lse.slurm.slurm_interfaces.sort import (
    run_full_sorting_pipeline_on_mouse,
)
from npix_lse.load_sync_channels.extract_sync_channel import extract_all_sync_channels
from npix_lse.slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def main():
    pipeline.show()


@magicgui
def pipeline(mouse_ids:list =["hello", "bye"],
             derivatives_directory: str ="/ceph/margrie/slenzi/2024/SC/derivatives/",
             rawdata_directory: str = "/ceph/margrie/slenzi/2024/SC/rawdata/",
             brainreg:bool =True,
             behaviour:bool =True,
             spikesorting:bool=True,
             bombcell:bool=True,
             ):

    for mouse_id in mouse_ids:
        mtd = SortingMetadata(
            mouse_id,
            root=derivatives_directory,
        )
        if not mtd.mouse_dir.exists():
            warnings.warn("mouse directory not found")
            raise ValueError

        print("syncing data...")
        sync_raw_and_processed_data(mouse_id=mtd.mouse_id,
                                    raw_directory=rawdata_directory,
                                    processed_directory=derivatives_directory)

        if brainreg:
            print("brainreg job setup...")
            brainreg_slurm(mtd)

        if behaviour:
            mtg = MouseLoomTrialGroup(mouse_id,  #necessary to initialise data
                                      processed_data_dir=derivatives_directory)

            print("tracking job setup...")
            track_mouse_slurm(mtd)

            print("extract sync channels...")
            extract_all_sync_channels(mtd)

        if spikesorting:
            print("sorting job setup...")
            run_full_sorting_pipeline_on_mouse(mtd)

        if bombcell:
            print("running bombcell...")
            bombcell_slurm(mtd.sessions[-1], overwrite=True)
            bombcell_all_sessions(mtd,
                                  overwrite=True
                                  )

if __name__ == "__main__":
    pipeline.show()
    print("")

    # mouse_ids = [
    #     "sub-016_id-1121662",
    # ]
    # derivatives_directory = pathlib.Path("/ceph/margrie/slenzi/2024/SC/derivatives/")
    # rawdata_directory = pathlib.Path("/ceph/margrie/slenzi/2024/SC/rawdata/")
    #
    # pipeline_kwargs = {
    #                     "brainreg": True,
    #                     "behaviour": True,
    #                     "spikesorting": True,
    #                     "bombcell": True,
    # }

# def bombcell_all_sessions(mtd, overwrite=False):
#     for s in mtd.sessions:
#         if s.kilosort_dir:
#             bombcell_slurm(s, overwrite)



bombcell_template_matlab_script= """
fprintf('running template bombcell script')\n
gain_to_uV = 0.195; \n
fprintf('bc_loadEphysData')\n

[spikeTimes_samples, spikeTemplates, templateWaveforms, templateAmplitudes, pcFeatures, pcFeatureIdx, channelPositions] = bc_loadEphysData(ephysKilosortPath);\n
fprintf('bc_loadEphysData done')\n
fprintf('bc_manageDataCompression')\n
rawFile = bc_manageDataCompression(ephysRawDir, decompressDataLocal);\n
fprintf('bc_manageDataCompression')\n

fprintf('bc_qualityParamValuesForUnitMatch')\n

%param = bc_qualityParamValues(ephysMetaDir, rawFile, ephysKilosortPath, gain_to_uV);\n 
param = bc_qualityParamValuesForUnitMatch(ephysMetaDir, rawFile); % Run this if you want to use UnitMatch after\n
fprintf('bc_qualityParamValuesForUnitMatch done')\n

rerun = 0; \n
fprintf('qmetrics')\n

qMetricsExist = ~isempty(dir(fullfile(savePath, 'qMetric*.mat'))) || ~isempty(dir(fullfile(savePath, 'templates._bc_qMetrics.parquet')));\n
\n


if qMetricsExist == 0 || rerun\n
    fprintf('running bc_runAllQualityMetrics')\n
    [qMetric, unitType] = bc_runAllQualityMetrics(param, spikeTimes_samples, spikeTemplates, templateWaveforms, templateAmplitudes,pcFeatures,pcFeatureIdx,channelPositions, savePath); \n
else\n
    fprintf('loading previous bc_runAllQualityMetrics')\n

    [param, qMetric] = bc_loadSavedMetrics(savePath);\n 
    unitType = bc_getQualityUnitType(param, qMetric, savePath);\n
end\n
\n
original_id_we_want_to_load = 0;\n
id_we_want_to_load_1_indexed = original_id_we_want_to_load + 1;\n 
number_of_spikes_for_this_cluster = qMetric.nSpikes(qMetric.clusterID == id_we_want_to_load_1_indexed);\n
original_id_we_want_to_load = 0;\n
number_of_spikes_for_this_cluster = qMetric.nSpikes(qMetric.phy_clusterID == original_id_we_want_to_load);\n
\n
goodUnits = unitType == 1;\n
muaUnits = unitType == 2;\n
noiseUnits = unitType == 0;\n
nonSomaticUnits = unitType == 3;\n 
\n
all_good_units_number_of_spikes = qMetric.nSpikes(goodUnits);\n
\n
label_table = table(unitType);\n
writetable(label_table,[savePath filesep 'templates._bc_unit_labels.tsv'],'FileType', 'text','Delimiter','\t');\n
\n
"""



import pathlib

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from looming_spots.loom_io.load import sync_raw_and_processed_data

from npix_lse.slurm.slurm_interfaces.bombcell import bombcell_slurm
from npix_lse.metadata.sorting_metadata import SortingMetadata
from npix_lse.slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def main():
    sub_ids = [
        #"sub-012_id-1121012",
        # "sub-013_id-1121013",
        # "sub-014_id-1121014",
        # "sub-016_id-1121016",
        # "sub-017_id-1121018",
        # "sub-025_id-1100581",
        # "sub-026_id-1100580",
        # "sub-028_id-1100589",
        #"sub-030_id-1100588",
        "sub-032_id-1100586",
        "sub-034_id-1100587",
        "sub-036_id-1100709",
        "sub-038_id-1100710",
        "sub-033_id-1100586_positive_control",
        "sub-035_id-1100587_positive_control",
        "sub-037_id-1100709_positive_control",
        "sub-039_id-1100710_positive_control",
    ]
    root = pathlib.Path("/ceph/margrie/slenzi/2023/julia_optostim/")
    rawdata_root = root / "rawdata"
    processed_data_root = root / "derivatives"

    for mouse_id in sub_ids:
        mtd = SortingMetadata(
            mouse_id,
            root=processed_data_root,
            probe=False,
        )

        sync_raw_and_processed_data(mouse_id=mtd.mouse_id,
                                    raw_directory=rawdata_root,
                                    processed_directory=processed_data_root,
                                    )

        mtg = MouseLoomTrialGroup(mouse_id,
                                  processed_data_dir=str(processed_data_root),
                                  photometry=False,
                                  )
        track_mouse_slurm(mtd)


def bombcell_all_sessions(mtd):
    for s in mtd.sessions:
        if s.kilosort_dir:
            bombcell_slurm(s)


if __name__ == "__main__":
    main()



from pp_slurm.slurm import run_job


def run_dlc_tracking_slurm(**kwargs) -> None:
    """
    Run the entire DLC pipeline in a SLURM job.

    Parameters
    ----------
    kwargs : Dict
        Keyword arguments passed to
        looming_spots.tracking_dlc.track_mouse.process_behaviour

    Notes
    -----
    The import must occur here to avoid recursive imports.
    """
    from lmtracker.track_mouse import process_behaviour

    run_job(kwargs, process_behaviour, "Track Mouse DLC")


def run_sorting_slurm(**kwargs) -> None:
    """
    Run the swc_ephys sorting pipeline from within a SLURM job.

    Notes
    -----
    The import must occur here to avoid recursive imports.
    """
    from ..pipeline.sort import run_sorting

    run_job(kwargs, run_sorting, "Sorting")


def run_brainreg_slurm(**kwargs) -> None:
    """
    Run brainreg in a SLURM job.

    Parameters
    ----------
    kwargs : Dict
        Keyword arguments passed to
        looming_spots.tracking_dlc.track_mouse.process_behaviour

    Notes
    -----
    The import must occur here to avoid recursive imports.
    """
    from breg_util.brainreg_command_line_util import brainreg_commandline

    run_job(kwargs, brainreg_commandline, "register brain to atlas")


def run_bombcell_slurm(**kwargs) -> None:
    """
    Run bombcell in a SLURM job.

    Parameters
    ----------
    kwargs :

    Notes
    -----
    The import must occur here to avoid recursive imports.
    """
    from bombcell.bombcell import bombcell_placeholder

    run_job(kwargs, bombcell_placeholder, "run bombcell for quality")



from pp_slurm import slurm_functions
from pathlib import Path


def track_mouse_slurm(mtd, skip_video_processing=False):
    kwarg_dict = {
        "slurm_batch": True,
        "logs_path": Path(f"{mtd.mouse_dir_derivatives}") / "logs",
        "mouse_id": f"{mtd.mouse_dir_derivatives.stem}",
        "track": True,
        "overwrite": True,
        "video_file_name": "camera",
        "input_video_fmt": "avi",
        "output_video_fmt": "mp4",
        "label_video": False,
        "rawdata_dir": mtd.mouse_dir_rawdata.parent,
        "derivatives_dir": mtd.mouse_dir_derivatives.parent,
        "module_string": "deeplabcut/2022-07-06",
        "additional_string": "export DLClight=True",
        "skip_video_processing": skip_video_processing,
    }
    slurm_functions.run_dlc_tracking_slurm(**kwarg_dict)



import os
from bombcell.bombcell_matlab_script_template import bombcell_template_matlab_script


def get_bombcell_paths_str(session_mtd):
    ephysKilosortPath = f'{session_mtd.kilosort_dir}'
    ephysRawDir = f'{session_mtd.raw_traces_path}'
    ephysMetaDir = f'{session_mtd.raw_meta_path}'
    savePath = f'{session_mtd.kilosort_dir.parent / "bombcell"}'
    decompressDataLocal = f'{session_mtd.kilosort_dir.parent / "bombcell_ephys"}'

    matlab_paths_string = f"ephysKilosortPath = '{ephysKilosortPath}'; \n" \
                          f"ephysRawDir = dir('{ephysRawDir}'); \n" \
                          f"ephysMetaDir = dir('{ephysMetaDir}'); \n" \
                          f"savePath = '{savePath}'; \n" \
                          f"decompressDataLocal = '{decompressDataLocal}'; \n"

    return matlab_paths_string


def write_bombcell_matlab_script(s, bombcell_location="/nfs/nhome/live/slenzi/code/bombcell"):
    bombcell_paths = get_bombcell_paths_str(s)
    script_content = f"addpath(genpath('{bombcell_location}')); \n{bombcell_paths} \n {bombcell_template_matlab_script}"
    file_name = s.kilosort_dir.parent / "bc_quality_metrics.m"
    with open(file_name, "w") as file:
        file.write(script_content)
        os.chmod(file_name, 0o755)


def bombcell_placeholder(logs_path=None):
    print('running_bombcell...')




"""
Here defaults related to the HPC system `spikewrap` is run on
(if not running locally) are stored.


`hpc_sorter_images_path()`
    This function returns the default path where singularity images are stored.
    The purpose of this storage path is to avoid users installing multiple copies
    across the system. This path can be changed based on the HPC system used and
    populated with SpikeInterface singularity images.

`default_slurm_options()`
    The default options passed to the submitit executor and converted
    to SLURM batch script arguments. If passing new options in `slurm_batch`,
    the passed options are overriden but other defaults are maintained.
"""
from typing import Dict


def hpc_sorter_images_path() -> str:
    return "/ceph/neuroinformatics/neuroinformatics/scratch/sorter_images"


def default_slurm_options() -> Dict:
    return {
        "nodes": 1,
        "mem_gb": 120,
        "timeout_min": 100 * 60,
        "cpus_per_task": 8,
        "tasks_per_node": 1,
        "gpus_per_node": 1,
        "slurm_gres": "gpu:1",
        "slurm_partition": "gpu",
        "slurm_exclude": "gpu-sr670-20,gpu-sr670-21,gpu-sr670-22",
        "wait": False,
        "env_name": "looming_spots",
    }


def default_gpu_partition():
    return "gpu"


def default_cpu_partition():
    return "cpu"



import pathlib

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from looming_spots.loom_io.load import sync_raw_and_processed_data

from npix_lse.metadata.sorting_metadata import SortingMetadata
from npix_lse.slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def main():
    sub_ids = [
    # "CA451A_2",
    # "CA451A_3",
    # "CA451A_4",
    # "CA507_1",
    # "CA507_3",
    # "CA507_4",
    # "CA493_1",
    # "CA493_2",
    # "CA459A_2",
    # "CA478_3",
    # "CA476_5",
    # "CA507_2",
    # "CA507_5",
    # "CA493_3",
    # "CA493_4",

    # "sub-064_id-1101073",
    # "sub-065_id-1101072",
    # "sub-066_id-1101057",
    # "sub-068_id-1101074",
    # "sub-067_id-1101075",
    # "sub-069_id-1101058",
    # "sub-072_id-1101189",
    # "sub-073_id-1101185",
    #"sub-076_id-1122311",
    #"sub-077_id-1101326",
    #"sub-078_id-1101447",
    "sub-079_id-1101446",
    ]
    root = pathlib.Path("/ceph/margrie/slenzi/loomer/")
    rawdata_root = root / "rawdata"
    processed_data_root = root / "derivatives"

    for mouse_id in sub_ids:
        mtd = SortingMetadata(
            mouse_id,
            root=processed_data_root,
            probe=False,
            make_mouse_dirs=False,
        )

        sync_raw_and_processed_data(mouse_id=mtd.mouse_id,
                                    raw_directory=rawdata_root,
                                    processed_directory=processed_data_root,
                                    )

        mtg = MouseLoomTrialGroup(mouse_id,
                                  processed_data_dir=str(processed_data_root),
                                  photometry=False,
                                  )
        track_mouse_slurm(mtd,
                          skip_video_processing=False)


if __name__ == "__main__":
    main()



from pp_slurm import slurm_functions
from pathlib import Path

from breg_util.brainreg_command_line_util import brainreg_command


def brainreg_slurm(mtd, additional=None):
    # This is a hack right now and passes the actually used command via the additional_string
    # command, the rest is more or less irrelevant.
    brainreg_commands = brainreg_command(
                                          mtd.mouse_dir_derivatives,
                                          mtd.serial2p_dir,
                                          function="brainreg",
                                          atlas="kim_mouse_10um",
                                          additional=additional
    )

    print(f"The following commands will be executed: {brainreg_commands}")

    for cmd in brainreg_commands:
        kwarg_dict = {
            "slurm_batch": True,
            "logs_path": Path(f"{mtd.mouse_dir_derivatives}") / "logs",
            "mouse_dir": f"{mtd.mouse_dir_derivatives}",
            "function": "brainreg",
            "atlas": "kim_mouse_10um",
            "module_string": "brainglobe",
            "additional_string": cmd,
        }
        slurm_functions.run_brainreg_slurm(**kwarg_dict)



import pathlib
import warnings

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from looming_spots.loom_io.load import sync_raw_and_processed_data

from npix_lse.metadata.sorting_metadata import SortingMetadata
from npix_lse.slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def main():
    # sub_ids = [
    #     #"sub-012_id-1121012", # corrupt photodiode for first recordings
    #     "sub-013_id-1121013",
    #     "sub-014_id-1121014",
    #     "sub-016_id-1121016",
    #     "sub-017_id-1121018",
    #     "sub-025_id-1100581",
    #     "sub-026_id-1100580",
    #     "sub-028_id-1100589",
    #     "sub-030_id-1100588",
    # ]
    rawdata_path = pathlib.Path("/ceph/margrie/juliaw/rawdata/optostim_experiments")
    derivatives_path = pathlib.Path("/ceph/margrie/juliaw/derivatives/optostim_experiments")

    #sub_ids = [x.stem for x in rawdata_path.glob("*")]
    sub_ids = [
        # "sub-055_id-1100827",
        # "sub-056_id-1100826",
        # "sub-057_id-1100828",
        # "sub-058_id-1100832",
        # "sub-058_id-1100833",
        # "sub-058_id-1100834",
        # "sub-060_id-1121712",
        # "sub-061_id-1121666",
        # "sub-062_id-1121667",

        #"sub-048_id-1121479",
        # "sub-053_id-1121567",
        # "sub-064_id-1100833",
        # "sub-063_id-1121667",
        # "sub-062_id-1121666",
        # "sub-061_id-1100834",
        # "sub-049_id-1121575",
        # "sub-046_id-1121577",
        # "sub-052_id-1121477",
        # "sub-048_id-1121479",
        # "sub-059_id-1121711",
        "sub-075_id-1101215",
        "sub-074_id-1101216",


    ]

    for mouse_id in sub_ids:
        mtd = SortingMetadata(
            mouse_id,
            root=derivatives_path,
            probe=False,
        )
        if not mtd.mouse_dir.exists():
            warnings.warn("mouse directory not found")
            raise ValueError

        try:
            sync_raw_and_processed_data(mouse_id=mtd.mouse_id, raw_directory=rawdata_path, processed_directory=derivatives_path)
            mtg = MouseLoomTrialGroup(mouse_id, processed_data_dir=derivatives_path)
            track_mouse_slurm(mtd)
        except Exception as e:
            print(e)
        #brainreg_slurm(mtd)


if __name__ == "__main__":
    main()



import datetime
import subprocess
from pathlib import Path
from typing import Callable, Dict

import submitit

from .hpc import (
    default_cpu_partition,
    default_gpu_partition,
    default_slurm_options,
)


def message_user(message: str) -> None:
    """
    Method to interact with user.

    Parameters
    ----------
    message : str
        Message to print.
    """
    print(f"\n{message}")


def system_call_success(command: str) -> bool:
    return (
        subprocess.run(
            command,
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ).returncode
        == 0
    )


def run_job(kwargs, command_func: Callable, command_name: str) -> None:
    """
    Run a job (e.g. run_full_pipeline, run_sorting) on SLURM.

    Parameters
    ----------
    kwargs : Dict
        Keyword arguments passed to run_full_pipeline.

    command_func : Callable
        The function to run (e.g. `run_full_pipeline()`.

    command_name : str
        The name of the command, typically command_func.__name__,
        formatted for logging.
    """
    passed_slurm_opts = kwargs.pop("slurm_batch")
    module_string = kwargs.pop("module_string")
    additional_string = kwargs.pop("additional_string")

    func_opts = kwargs

    slurm_opts = default_slurm_options()
    env_name = slurm_opts.pop("env_name")

    if isinstance(passed_slurm_opts, Dict):
        slurm_opts.update(passed_slurm_opts)

    should_wait = slurm_opts.pop("wait")

    executor = get_executor(func_opts, slurm_opts)

    job = executor.submit(
        wrap_function_with_env_and_module_setup,
        command_func,
        env_name,
        module_string,
        additional_string,
        func_opts,
    )

    if should_wait:
        job.wait()

    send_user_start_message(command_name, job, func_opts)


# Utils --------------------------------------------------------------------------------


def get_executor(func_opts: Dict, slurm_opts: Dict) -> submitit.AutoExecutor:
    """
    Return the executor object that defines parameters
    of the SLURM node to request and the path to
    logs.

    Parameters
    ----------
    func_opts : Dict
        All arguments passed to the public function, minus
        `slurm_batch`

    slurm_opts : Dict
        The pp_slurm options to run. This includes `spikewarp` default
        pp_slurm options overwritten where passed by user-defined
        `slurm_batch`.

    Returns
    -------
    executor : submitit.AutoExecutor
        submitit executor object defining requested SLURM
        node parameters.
    """
    log_path = make_job_log_output_path(func_opts)

    print(f"\nThe SLURM batch output logs will " f"be saved to {log_path}\n")

    executor = submitit.AutoExecutor(
        folder=log_path,
    )

    executor.update_parameters(**slurm_opts)

    return executor


def wrap_function_with_env_setup(
    function: Callable, env_name: str, func_opts: Dict
) -> None:
    """
    Set up the environment from within the SLURM job, prior
    to running the processing function (e.g. run_full_pipeline).

    This is required to set up the conda environment within the job
    or the processing function will fail.

    Parameters
    ----------
    function : Callable
        The ephys processing function to run in the SLURM job
        e.g. run_full_pipeline, run_sorting

    slurm_opts : Union[Literal[True], Dict]
        A kwarg passed to the processing function (e.g. run_full_pipeline)
        indicating whether to run in the SLURM job. If True or a Dict,
        the SLURM job is run. If a dict, the environment setup
        can be passed in the 'env_name' field.

    func_opts : Dict
        All arguments passed to the public function, minus
        `slurm_batch`
    """
    print(f"\nrunning {function.__name__} with SLURM....\n")

    subprocess.run(
        f"module load deeplabcut/2023-08-01; "
        f"source activate {env_name}; "
        f"export DLClight=True",
        executable="/bin/bash",
        shell=True,
    )

    function(**func_opts)


def wrap_function_with_env_and_module_setup(
    function: Callable,
    env_name: str,
    module_string: str,
    additional_string: str,
    func_opts: Dict,
) -> None:
    """
    Set up the environment from within the SLURM job, prior
    to running the processing function (e.g. run_full_pipeline).

    This is required to set up the conda environment within the job
    or the processing function will fail.

    Parameters
    ----------
    function : Callable
        The ephys processing function to run in the SLURM job
        e.g. run_full_pipeline, run_sorting

    slurm_opts : Union[Literal[True], Dict]
        A kwarg passed to the processing function (e.g. run_full_pipeline)
        indicating whether to run in the SLURM job. If True or a Dict,
        the SLURM job is run. If a dict, the environment setup
        can be passed in the 'env_name' field.

    func_opts : Dict
        All arguments passed to the public function, minus
        `slurm_batch`
    """
    print(f"\nrunning {function.__name__} with SLURM....\n")

    subprocess.run(
        f"module load {module_string}; "
        f"source activate {env_name}; "
        f"{additional_string}",
        executable="/bin/bash",
        shell=True,
    )

    function(**func_opts)


def make_job_log_output_path(func_opts: Dict) -> Path:
    """
    The SLURM job logs are saved to a folder 'slurm_logs' in the
    base directory in which the processing is being run
    (i.e. the folder containing rawdata, derivatives). .

    Parameters
    ----------
    func_opts : Dict
        Keyword arguments passed to the main running function
        (e.g. run_full_pipeline, run_sorting)

    Returns
    -------
    log_path : Path
        The path to the SLURM log output folder for the current job.
        The logs are saved to a folder with machine datetime.
    """
    now = datetime.datetime.now()

    log_subpath = Path("slurm_logs") / f"{now.strftime('%Y-%m-%d_%H-%M-%S')}"

    if "base_path" in func_opts:
        log_path = func_opts["base_path"] / log_subpath
    else:
        # in the case of `run_preprocess()`, the
        # `PreprocessingData` object is passed.
        log_path = func_opts["logs_path"] / log_subpath

    log_path.mkdir(exist_ok=True, parents=True)

    return log_path


def send_user_start_message(
    processing_function: str, job: submitit.Job, func_opts: Dict
) -> None:
    """
    Convenience function to print important information
    regarding the SLURM job.

    Parameters
    ----------
    processing_function : str
        The function being run (i.e. run_full_pipeline, run_sorting)

    job : submitit.job
        submitit.job object holding the SLURM job_id

    func_opts : Dict
        Keyword arguments passed to the main running function
        (e.g. run_full_pipeline, run_sorting)
    """
    message_user(
        f"{processing_function} submitted to SLURM with job id {job.job_id}\n"
        f"with arguments{func_opts}"
    )


def is_slurm_installed():
    slurm_installed = system_call_success("sinfo -v")
    return slurm_installed


def run_interactive_slurm(partition=None, gpu=True, mem_gb="40GB", cpus=16):
    """
    A convenience function to start an interactive SLURM session. Gives quick
    access for some basic settings - the purpose is not to provide a comprehensive
    wrapper, of course native `srun` can be called. It is only to provide a
    quick convenience wrapper to get started quick with some sensible defaults.
    """
    if not is_slurm_installed():
        raise RuntimeError(
            "Cannot setup interactive SLURM because SLURM is not "
            "installed on this machine."
        )

    if partition is None:
        partition = default_gpu_partition() if gpu else default_cpu_partition()

    gpu_opt = "--gres=gpu:1" if gpu else ""

    default_opts = default_slurm_options()
    exclude = default_opts["exclude"]

    command = f"srun -p {partition} {gpu_opt} -n {cpus} --mem={mem_gb} --exclude {exclude} --pty bash -i"

    subprocess.run(command, shell=True)



import fire
from magicgui import magicgui
from qtpy.QtWidgets import QApplication, QWidget, QVBoxLayout
from pathlib import Path
import warnings

from looming_spots.db.loom_trial_group import MouseLoomTrialGroup
from daq_loader.load import sync_raw_and_processed_data
from pp_slurm.slurm_interfaces.bombcell import bombcell_slurm
from pp_slurm.slurm_interfaces.brainreg import brainreg_slurm
from xpmtd.sorting_metadata import SortingMetadata
from pp_slurm.slurm_interfaces.sort import run_full_sorting_pipeline_on_mouse
from probe_loader.load_probe import extract_all_sync_channels
from pp_slurm.slurm_interfaces.track_mouse_dlc import track_mouse_slurm


def bombcell_all_sessions(mtd, overwrite=False):
    for s in mtd.sessions:
        if s.kilosort_dir:
            bombcell_slurm(s, overwrite)


@magicgui(
    rawdata_directory={"mode": "d"},
    derivatives_directory={"mode": "d"},
    serial2p_dir={"mode": "d"},
    mouse_ids={"widget_type": "Select", "choices": [], "allow_multiple": True},  # Multi-select
)
def pipeline_widget(
                    rawdata_directory=Path("/ceph/margrie/slenzi/2024/SC/rawdata/"),
                    derivatives_directory=Path("/ceph/margrie/slenzi/2024/SC/derivatives/"),
                    serial2p_dir=Path("/ceph/margrie/slenzi/serial2p/whole_brains/raw/"),
                    mouse_ids=[],
                    brainreg: bool = False,
                    behaviour: bool = False,
                    spikesorting: bool = False,
                    bombcell: bool = False,
                    tracking: bool = False,
):
    print(f"The following mouse IDs will now be analysed: {mouse_ids}")
    for mouse_id in mouse_ids:
        print(f"Processing mouse ID: {mouse_id}")
        print(serial2p_dir)

        mtd = SortingMetadata(mouse_id,
                              rawdata_directory=rawdata_directory,
                              derivatives_directory=derivatives_directory,
                              serial2p_dir=serial2p_dir,
                              behaviour=behaviour,
                              probe=spikesorting)

        if not mtd.mouse_dir_rawdata.exists():
            warnings.warn("mouse directory not found")
            raise ValueError

        print("syncing data...")
        sync_raw_and_processed_data(mouse_id=mtd.mouse_id,
                                    raw_directory=rawdata_directory,
                                    processed_directory=derivatives_directory)

        if brainreg:
            print("brainreg job setup...")
            brainreg_slurm(mtd)

        if behaviour:
            print("processing behaviour nidaq data...")
            mtg = MouseLoomTrialGroup(mouse_id, processed_data_dir=derivatives_directory)

            print("extract sync channels...")
            extract_all_sync_channels(mtd)

        if tracking:
            print("tracking job setup...")
            track_mouse_slurm(mtd)

        if spikesorting:
            print("sorting job setup...")
            run_full_sorting_pipeline_on_mouse(mtd)

        if bombcell:
            print("running bombcell...")
            bombcell_slurm(mtd.sessions[-1], overwrite=True)
            bombcell_all_sessions(mtd, overwrite=True)


def load_experiment_directories(event=None):
    p = pipeline_widget.rawdata_directory.value
    if p is None:
        print("No directory selected")
        return
    paths = list(Path(p).glob("*"))
    mouse_ids = [path.stem for path in paths if path.is_dir()]
    print("Mouse IDs:", mouse_ids)
    pipeline_widget.mouse_ids.choices = mouse_ids  # Update choices with mouse IDs


pipeline_widget.rawdata_directory.changed.connect(load_experiment_directories)


def pipelinerz_gui():
    app = QApplication([])
    widget = pipeline_widget
    main_window = QWidget()

    layout = QVBoxLayout()
    layout.addWidget(widget.native)

    main_window.setLayout(layout)
    main_window.setWindowTitle("Pipeline Widget")
    main_window.show()

    app.exec_()


def main():
    fire.Fire(pipelinerz_gui())


if __name__ == "__main__":
    main()
